@@ -12,7 +12,7 @@ from django.db.models.query import QuerySet, ValuesQuerySet, ValuesListQuerySet,
 from django.db.models.signals import post_save, post_delete, m2m_changed
 from django.utils.hashcompat import md5_constructor
 
-from cacheops.conf import model_profile, redis_conn
+from cacheops.conf import model_profile, redis_client
 from cacheops.utils import monkey_mix, dnf, conj_scheme, get_model_name
 from cacheops.invalidation import cache_schemes, conj_cache_key, invalidate_obj, invalidate_model
 
@@ -35,7 +35,7 @@ def cache_thing(model, cache_key, data, cond_dnf=[[]], timeout=None):
     schemes = map(conj_scheme, cond_dnf)
     cache_schemes.ensure_known(model, schemes)
 
-    txn = redis_conn.pipeline()
+    txn = redis_client.pipeline()
 
     # Write data to cache
     pickled_data = pickle.dumps(data, -1)
@@ -82,18 +82,17 @@ def cached_as(sample, extra=None, timeout=None):
         else:
             key_extra = '%s.%s' % (func.__module__, func.__name__)
         cache_key = queryset._cache_key(extra=key_extra)
-        cond_dnf = dnf(queryset.query.where, queryset.model._meta.db_table)
 
         @wraps(func)
         def wrapper(*args):
             # NOTE: These args must not effect function result.
             #       I'm keeping them to cache view functions.
-            cache_data = redis_conn.get(cache_key)
+            cache_data = redis_client.get(cache_key)
             if cache_data is not None:
                 return pickle.loads(cache_data)
 
             result = func(*args)
-            cache_thing(queryset.model, cache_key, result, cond_dnf, timeout or queryset._cachetimeout)
+            queryset._cache_results(cache_key, result, timeout)
             return result
 
         return wrapper
@@ -188,7 +187,8 @@ class QuerySetMixin(object):
     def __init__(self, *args, **kwargs):
         self._no_monkey.__init__(self, *args, **kwargs)
         self._cloning = 1000
-        if not hasattr(self, '_cacheprofile') and self.model:
+        self._cacheprofile = None
+        if self.model:
             self._cacheprofile = model_profile(self.model)
             self._cache_write_only = False
             if self._cacheprofile is not None:
@@ -226,9 +226,9 @@ class QuerySetMixin(object):
 
         return 'q:%s' % md5.hexdigest()
 
-    def _cache_results(self, cache_key, results):
-        cond_dnf = dnf(self.query.where, self.model._meta.db_table)
-        cache_thing(self.model, cache_key, results, cond_dnf, timeout=self._cachetimeout)
+    def _cache_results(self, cache_key, results, timeout=None):
+        cond_dnf = dnf(self)
+        cache_thing(self.model, cache_key, results, cond_dnf, timeout or self._cachetimeout)
 
     def cache(self, ops=None, timeout=None, write_only=None):
         """
@@ -303,13 +303,13 @@ class QuerySetMixin(object):
 
     def iterator(self):
         superiter = self._no_monkey.iterator
-        cache_this = self._cacheprofile is not None and 'fetch' in self._cacheops
+        cache_this = self._cacheprofile and 'fetch' in self._cacheops
 
         if cache_this:
             cache_key = self._cache_key()
             if not self._cache_write_only:
                 # Trying get data from cache
-                cache_data = redis_conn.get(cache_key)
+                cache_data = redis_client.get(cache_key)
                 if cache_data is not None:
                     results = pickle.loads(cache_data)
                     for obj in results:
@@ -337,7 +337,7 @@ class QuerySetMixin(object):
     def get(self, *args, **kwargs):
         # .get() uses the same .iterator() method to fetch data,
         # so here we add 'fetch' to ops
-        if self._cacheprofile is not None and 'get' in self._cacheops:
+        if self._cacheprofile and 'get' in self._cacheops:
             # NOTE: local_get=True enables caching of simple gets in local memory,
             #       which is very fast, but not invalidated.
             # Don't bother with Q-objects, select_related and previous filters,
@@ -371,8 +371,8 @@ class QuerySetMixin(object):
               Yes, if you use .exists() yourself this can cause memory leak.
         """
         # TODO: refactor this one to more understandable something
-        if self._cacheprofile is not None:
-            query_dnf = dnf(self.query.where, self.model._meta.db_table)
+        if self._cacheprofile:
+            query_dnf = dnf(self)
             if len(query_dnf) == 1 and len(query_dnf[0]) == 1 and query_dnf[0][0][0] == self.model._meta.pk.name:
                 result = len(self.nocache()) > 0
                 if result:
@@ -466,13 +466,21 @@ def invalidate_m2m(sender=None, instance=None, model=None, action=None, pk_set=N
     if action in ('post_add', 'post_remove', 'post_clear'):
         invalidate_model(sender) # NOTE: this is harsh, but what's the alternative?
         invalidate_obj(instance)
-        # TODO: we should invalidate referenced objects as well
+        # TODO: more granular invalidation for referenced models
+        invalidate_model(model)
 
 
+installed = False
+
 def install_cacheops():
     """
     Installs cacheops by numerous monkey patches
     """
+    global installed
+    if installed:
+        return # just return for now, second call is probably done due cycle imports
+    installed = True
+
     monkey_mix(Manager, ManagerMixin)
     monkey_mix(QuerySet, QuerySetMixin)
     monkey_mix(ValuesQuerySet, QuerySetMixin, ['iterator'])
@@ -485,15 +493,17 @@ def install_cacheops():
         model._default_manager._install_cacheops(model)
 
     # Turn off caching in admin
-    from django.contrib.admin.options import ModelAdmin
-    def ModelAdmin_queryset(self, request):
-        queryset = o_ModelAdmin_queryset(self, request)
-        if queryset._cacheprofile is None:
-            return queryset
-        else:
-            return queryset.nocache()
-    o_ModelAdmin_queryset = ModelAdmin.queryset
-    ModelAdmin.queryset = ModelAdmin_queryset
+    from django.conf import settings
+    if 'django.contrib.admin' in settings.INSTALLED_APPS:
+        from django.contrib.admin.options import ModelAdmin
+        def ModelAdmin_queryset(self, request):
+            queryset = o_ModelAdmin_queryset(self, request)
+            if queryset._cacheprofile is None:
+                return queryset
+            else:
+                return queryset.nocache()
+        o_ModelAdmin_queryset = ModelAdmin.queryset
+        ModelAdmin.queryset = ModelAdmin_queryset
 
     # bind m2m changed handler
     m2m_changed.connect(invalidate_m2m)
